/* eslint-disable class-methods-use-this */
import hwSdk from "./hwSdk";
import HyperwalletTransaction from "./HyperwalletTransaction";

const usersURL = "users/";
const transferMethodURL = "/transfer-methods";
const businessStakeholdersURL = "/business-stakeholders";
const statusTransitionsURL = "/status-transitions";

export default class Hyperwallet {
    getUser(callback) {
        hwSdk.getHyperwalletConfiguration((error, hyperwalletConfiguration) => {
            if (error) {
                callback(error);
            }
            const { restUri } = hyperwalletConfiguration;
            const { walletToken } = hyperwalletConfiguration;
            HyperwalletTransaction.get({
                url: `${restUri}${usersURL}${walletToken}`,
                authenticationToken: hyperwalletConfiguration.authenticationToken,
                callback,
            });
        });
    }

    updateUser(data, callback) {
        hwSdk.getHyperwalletConfiguration((error, hyperwalletConfiguration) => {
            if (error) {
                callback(error);
            }
            const { restUri } = hyperwalletConfiguration;
            const { walletToken } = hyperwalletConfiguration;
            HyperwalletTransaction.put({
                url: `${restUri}${usersURL}${walletToken}`,
                authenticationToken: hyperwalletConfiguration.authenticationToken,
                data,
                callback,
            });
        });
    }

    uploadUserDocuments(jsonData, callback) {
        hwSdk.getHyperwalletConfiguration((error, hyperwalletConfiguration) => {
            if (error) {
                callback(error);
            }
            const { restUri } = hyperwalletConfiguration;
            const { walletToken } = hyperwalletConfiguration;
            HyperwalletTransaction.put({
                url: `${restUri}${usersURL}${walletToken}`,
                authenticationToken: hyperwalletConfiguration.authenticationToken,
                data: jsonData.data,
                files: jsonData.files,
                callback,
            });
        });
    }

    getTransferMethod(transferMethodToken, callback) {
        hwSdk.getHyperwalletConfiguration((error, hyperwalletConfiguration) => {
            if (error) {
                callback(error);
            }
            const { restUri } = hyperwalletConfiguration;
            const { walletToken } = hyperwalletConfiguration;
            HyperwalletTransaction.get({
                url: `${restUri}${usersURL}${walletToken}${transferMethodURL}/${transferMethodToken}`,
                authenticationToken: hyperwalletConfiguration.authenticationToken,
                callback,
            });
        });
    }

    updateTransferMethod(transferMethodToken, data, callback) {
        hwSdk.getHyperwalletConfiguration((error, hyperwalletConfiguration) => {
            if (error) {
                callback(error);
            }
            const { restUri } = hyperwalletConfiguration;
            const { walletToken } = hyperwalletConfiguration;
            HyperwalletTransaction.put({
                url: `${restUri}${usersURL}${walletToken}${transferMethodURL}/${transferMethodToken}`,
                authenticationToken: hyperwalletConfiguration.authenticationToken,
                data,
                callback,
            });
        });
    }

    createTransferMethod(data, callback) {
        hwSdk.getHyperwalletConfiguration((error, hyperwalletConfiguration) => {
            if (error) {
                callback(error);
            }
            const { restUri } = hyperwalletConfiguration;
            const { walletToken } = hyperwalletConfiguration;
            HyperwalletTransaction.post({
                url: `${restUri}${usersURL}${walletToken}${transferMethodURL}`,
                authenticationToken: hyperwalletConfiguration.authenticationToken,
                data,
                callback,
            });
        });
    }

    listStakeholders(isDirector, isUltimateBeneficialOwner, isBusinessContact, callback) {
        hwSdk.getHyperwalletConfiguration((error, hyperwalletConfiguration) => {
            if (error) {
                callback(error);
            }
            const { restUri } = hyperwalletConfiguration;
            const { walletToken } = hyperwalletConfiguration;
            let params = "";
            if (isDirector !== undefined) {
                params += `&isDirector=${isDirector}`;
            }
            if (isUltimateBeneficialOwner !== undefined) {
                params += `&isUltimateBeneficialOwner=${isUltimateBeneficialOwner}`;
            }
            if (isBusinessContact !== undefined) {
                params += `&isBusinessContact=${isBusinessContact}`;
            }
            HyperwalletTransaction.get({
                url: `${restUri}${usersURL}${walletToken}${businessStakeholdersURL}?sortBy=-createdOn&status=ACTIVATED${params}`,
                authenticationToken: hyperwalletConfiguration.authenticationToken,
                callback,
            });
        });
    }

    getStakeholder(stakeholderToken, callback) {
        hwSdk.getHyperwalletConfiguration((error, hyperwalletConfiguration) => {
            if (error) {
                callback(error);
            }
            const { restUri } = hyperwalletConfiguration;
            const { walletToken } = hyperwalletConfiguration;
            HyperwalletTransaction.get({
                url: `${restUri}${usersURL}${walletToken}${businessStakeholdersURL}/${stakeholderToken}`,
                authenticationToken: hyperwalletConfiguration.authenticationToken,
                callback,
            });
        });
    }

    createStakeholder(data, callback) {
        hwSdk.getHyperwalletConfiguration((error, hyperwalletConfiguration) => {
            if (error) {
                callback(error);
            }
            const { restUri } = hyperwalletConfiguration;
            const { walletToken } = hyperwalletConfiguration;
            HyperwalletTransaction.post({
                url: `${restUri}${usersURL}${walletToken}${businessStakeholdersURL}`,
                authenticationToken: hyperwalletConfiguration.authenticationToken,
                data,
                callback,
            });
        });
    }

    updateStakeholder(stakeholderToken, data, callback) {
        hwSdk.getHyperwalletConfiguration((error, hyperwalletConfiguration) => {
            if (error) {
                callback(error);
            }
            const { restUri } = hyperwalletConfiguration;
            const { walletToken } = hyperwalletConfiguration;
            HyperwalletTransaction.put({
                url: `${restUri}${usersURL}${walletToken}${businessStakeholdersURL}/${stakeholderToken}`,
                authenticationToken: hyperwalletConfiguration.authenticationToken,
                data,
                callback,
            });
        });
    }

    uploadStakeholderDocuments(stakeholderToken, jsonData, callback) {
        hwSdk.getHyperwalletConfiguration((error, hyperwalletConfiguration) => {
            if (error) {
                callback(error);
            }
            const { restUri } = hyperwalletConfiguration;
            const { walletToken } = hyperwalletConfiguration;
            HyperwalletTransaction.put({
                url: `${restUri}${usersURL}${walletToken}${businessStakeholdersURL}/${stakeholderToken}`,
                authenticationToken: hyperwalletConfiguration.authenticationToken,
                data: jsonData.data,
                files: jsonData.files,
                callback,
            });
        });
    }

    createUserStakeholderStatusTransition(stakeholderToken, statusTransition, callback) {
        hwSdk.getHyperwalletConfiguration((error, hyperwalletConfiguration) => {
            if (error) {
                callback(error);
            }
            const { restUri } = hyperwalletConfiguration;
            const { walletToken } = hyperwalletConfiguration;
            HyperwalletTransaction.post({
                url: `${restUri}${usersURL}${walletToken}${businessStakeholdersURL}/${stakeholderToken}${statusTransitionsURL}`,
                authenticationToken: hyperwalletConfiguration.authenticationToken,
                data: {
                    transition: statusTransition,
                },
                callback,
            });
        });
    }
}
