
export default class HyperwalletConfiguration {
    constructor(retrieveAuthenticationToken) {
        this.retrieveAuthenticationToken = retrieveAuthenticationToken;
    }

    getAuthenticationToken(callback) {
        if (this.isAuthenticationTokenValid(this.authenticationToken)) {
            callback(this.authenticationToken);
        } else {
            this.loadAuthenticationToken(callback);
        }
    }

    loadAuthenticationToken(callback) {
        if (this.isSetup()) {
            this.retrieveAuthenticationToken(
                (validAuthenticationToken) => this.onSuccess(validAuthenticationToken, callback),
                (error) => HyperwalletConfiguration.onFailure(error, callback)
            );
        } else {
            throw new Error("HWWidgets.initialize(retrieveAuthenticationToken) was not provided.");
        }
    }

    isSetup() {
        return this.retrieveAuthenticationToken !== undefined && this.retrieveAuthenticationToken !== null;
    }


    isAuthenticationTokenValid() {
        if (this.authenticationToken === undefined) {
            return false;
        }
        const currentTime = Math.floor(Date.now() / 1000);
        const creationTime = this.createdOn;
        const expiryTime = this.expiresOn;
        const bufferFraction = 0.1; // this value may change later
        const bufferTime = bufferFraction * (expiryTime - creationTime);
        const staleOrExpired = currentTime + bufferTime > expiryTime;
        if (staleOrExpired) {
            return false;
        }
        return true;
    }

    onSuccess(validAuthenticationToken, callback) {
        this.setAuthenticationToken(validAuthenticationToken);
        callback(this.authenticationToken);
    }

    static onFailure(error, callback) {
        callback(null, error);
    }

    setAuthenticationToken(validAuthenticationToken) {
        this.authenticationToken = validAuthenticationToken;
        this.internalParsePayload(validAuthenticationToken);
    }

    internalParsePayload(authenticationToken) {
        const payloadEncoded = authenticationToken.split(".")[1];
        const payloadJson = JSON.parse(atob(payloadEncoded));
        this.createdOn = payloadJson.iat;
        this.expiresOn = payloadJson.exp;
        this.restUri = HyperwalletConfiguration.getRestUri(payloadJson["rest-uri"]);
        this.graphqlUri = payloadJson["graphql-uri"];
        this.issuerToken = payloadJson.iss;
        this.walletToken = payloadJson.sub;
        this.userToken = payloadJson.pgu;
    }

    static getRestUri(rawRestUri) {
        // temp fix HW-56838 as business-stakeholders endpoints are not accessible through v3
        return rawRestUri.replace(/(.+)\/rest\/\w{2}\//, (...match) => `${match[1]}/rest/v4/`);
    }
}
